/*
 * Simple program that sniffs various packets from the net.
 * Only for study ;-)
 * 
 * by: LexLufix
 * Lex@Lufix.net
 * http://www.lufix.net/
 *
 * */

#include "nsniff.h"

void *ip6_process(char *ptr, int len) {
		int hlen;
		struct ip6_hdr *ip6;
		struct hostent *hptr;
		
		if (len < sizeof(struct ip6_hdr)) {
				printf("hmm, looks to small for an IPv6 packet. skipping...");
				return;
		}
		printf("\nIPv6-packet cached. ");
		ip6 = (struct ip6_hdr *) ptr;
		hlen = sizeof(struct ip6_hdr);
		if (ip6->ip6_flow >> 4 != 6) {
				printf("...but IP version (%d) != 6. skipping...", ip6->ip6_flow >> 4);
				return;
		}
		print_ip6addr((char *) &ip6->ip6_src);
		printf(" >> ");
		print_ip6addr((char *) &ip6->ip6_dst);
		printf(". ");
		
		if (verbose) {
				printf("priority: %d; ", ip6->ip6_flow >> 8);
				printf("flow-label: %d; ", ip6->ip6_flow + 1);
				printf("hop-limit: %d. ", ip6->ip6_hlim);
		}
		switch (ip6->ip6_nxt) {
				case IPPROTO_HOPOPTS:
						ip6hopopts_process(ptr + hlen);
						break;
				case IPPROTO_DSTOPTS:
						ip6dstopts_process(ptr + hlen);
						break;
				case IPPROTO_FRAGMENT:
						ip6frag_process(ptr + hlen);
						break;
				case IPPROTO_ROUTING:
						ip6rout_process(ptr + hlen);
						break;
						
				case IPPROTO_TCP:
						tcp_process(ptr + hlen);
						break;
				case IPPROTO_UDP:
						udp_process(ptr + hlen);
						break;
				case IPPROTO_ICMPV6:
						icmp6_process(ptr + hlen);
						break;
				default:
						printf("packet contains an unknown protocol, skipping...");
						checkcount();
						break;
		}
		
		return;
}

void *ip6hopopts_process(char *ptr) {
		return;
}
void *ip6dstopts_process(char *ptr) {
		return;
}
void *ip6rout_process(char *ptr) {
		return;
}
void *ip6frag_process(char *ptr) {
		return;
}

void icmp6_process(char *ptr) {
		struct icmp6_hdr *icmp6;
		int icmp6t, icmp6c;
    
		icmp6 = (struct icmp6_hdr *) ptr;
		icmp6t = icmp6->icmp6_type;
		icmp6c = icmp6->icmp6_code;
		
		printf("\nProtocol: ICMPv6. ");
		switch (icmp6t) {
				case ICMP6_DST_UNREACH:
						printf("destination unreachable (type %d): ", icmp6t);
						switch (ntohs(icmp6c)) {
								case ICMP6_DST_UNREACH_NOROUTE:
										printf("no route to destination (code %d).", icmp6c);
										break;
								case ICMP6_DST_UNREACH_ADMIN:
										printf("administratively prohibited (code %d).", icmp6c);
										break;
								case ICMP6_DST_UNREACH_NOTNEIGHBOR:
										printf("not a neighbor (code %d).", icmp6c);
										break;
								case ICMP6_DST_UNREACH_ADDR:
										printf("address not reachable (code %d).", icmp6c);
										break;
								case ICMP6_DST_UNREACH_NOPORT:
										printf("port unreachable (code %d).", icmp6c);
										break;
								default:
										printf("unknown code field (%d) in ICMPv6 header.", icmp6c);
										break;
						}
						break;
						
				case ICMP6_PACKET_TOO_BIG:
						printf("packet too big (type %d). ", icmp6t);
						break;

				case ICMP6_TIME_EXCEEDED:
						printf("time exceeded (type %d): ", icmp6t);
						switch (icmp6c) {
								case ICMP6_TIME_EXCEED_TRANSIT:
										printf("in transit (code %d).", icmp6c);
										break;
								case ICMP6_TIME_EXCEED_REASSEMBLY:
										printf("fragment reassembly time exceeded (code %d).", icmp6c);
										break;
								default:
										printf("unknown code field (%d) in ICMPv6 header.", icmp6c);
										break;
						}
						break;
						
				case ICMP6_PARAM_PROB:
						printf("parameter problem (type %d): ", icmp6t);
						switch (icmp6c) {
								case ICMP6_PARAMPROB_HEADER:
										printf("errorneous header field (code %d).", icmp6c);
										break;
								case ICMP6_PARAMPROB_NEXTHEADER:
										printf("unrecognized next header (code %d).", icmp6c);
										break;
								case ICMP6_PARAMPROB_OPTION:
										printf("unrecognized option (code %d).", icmp6c);
										break;
								default:
										printf("unknown code field (%d) in ICMPv6 header.", icmp6c);
										break;
						}
						break;
						
				case ICMP6_ECHO_REQUEST:
						printf("echo request (type %d).", icmp6t);
						break;
				case ICMP6_ECHO_REPLY:
						printf("echo reply (type %d).", icmp6t);
						break;
				case ICMP6_MEMBERSHIP_QUERY:
						printf("group membership query (type %d).", icmp6t);
						break;
				case ICMP6_MEMBERSHIP_REPORT:
						printf("group membership report (type %d).", icmp6t);
						break;
				case ICMP6_MEMBERSHIP_REDUCTION:
						printf("group membership reduction (type %d).", icmp6t);
						break;
				case ND_ROUTER_SOLICIT:
						printf("router solicitation. (type %d).", icmp6t);
						break;
				case ND_ROUTER_ADVERT:
						printf("router advertisement. (type %d).", icmp6t);
						break;
				case ND_NEIGHBOR_SOLICIT:
						printf("neighbor solicitation (type %d).", icmp6t);
						break;
				case ND_NEIGHBOR_ADVERT:
						printf("neighbor advertisement (type %d).", icmp6t);
						break;
				case ND_REDIRECT:
						printf("redirect (type %d).", icmp6t);
						break;

				default:
						printf("unknown type field (%d) in ICMPv6 header.", icmp6t);
						break;	
		}
		checkcount();	
				
		return;
}

