/*
 * Simple program that sniffs various packets from the net.
 * Only for study ;-)
 * 
 * by: LexLufix
 * Lex@Lufix.net
 * http://www.lufix.net/
 *
 * */

#include "nsniff.h"

void checkport(uint16_t source, uint16_t dest, char *ptr) {
		if (source==PORT_DNS || dest==PORT_DNS)
				dns_process(ptr);
		else
				checkcount();

		return;
}

void *ip_process(char *ptr, int len) {
		int hlen;
		struct ip *ip;
		struct hostent *hptr;
		
		printf("\nIP-packet cached. ");
		if (len < sizeof(struct ip)) {
				printf("hmm, looks to small for an IP packet. skipping...");
				return;
		}
		ip = (struct ip *) ptr;
		if (ip->ip_v != 4) {
				printf("...but IP version (%d) != 4. skipping...", ip->ip_v);
				return;
		}
		hlen = ip->ip_hl << 2;
		if (hlen < sizeof(struct ip)) {
				printf("hmm, the header is to small (%d) for an IP packet. skipping...\n", hlen);
				return;
		}		
		if (verbose)
				printf("IP-total-length is %d. ", ntohs(ip->ip_len));
		if (names) {
				if ((hptr = gethostbyaddr(&ip->ip_src, 4, AF_INET)) == NULL)
						printf("%s >>", inet_ntoa(ip->ip_src));
				else
						printf("%s >>", hptr->h_name);
				if ((hptr = gethostbyaddr(&ip->ip_dst, 4, AF_INET)) == NULL)
						printf(" %s.", inet_ntoa(ip->ip_dst));
				else
						printf(" %s .", hptr->h_name);
		} else {
				printf("%s >>", inet_ntoa(ip->ip_src));
				printf(" %s.", inet_ntoa(ip->ip_dst));
		}
		if (verbose) {
				printf(" TTL: %d, ID: %d,", ip->ip_ttl, ip->ip_id);
				switch (ip->ip_tos) {
						case IPTOS_TOS_MASK:
								printf(" TOS: mask");
								break;
						case IPTOS_LOWDELAY:
								printf(" TOS: LowDelay");
								break;
						case IPTOS_THROUGHPUT:
								printf(" TOS: MaxThroughput");
								break;
						case IPTOS_RELIABILITY:
								printf(" TOS: MaxReliability");
								break;
						case IPTOS_LOWCOST:
								printf(" TOS: LowCost");
								break;
						default:
								printf(" TOS not specified");
								break;
				}
		}
		
		switch (ip->ip_p) {
				case IPPROTO_TCP:
						tcp_process(ptr + hlen);
						break;
				case IPPROTO_UDP:
						udp_process(ptr + hlen);
						break;
				case IPPROTO_ICMP:
						icmp_process(ptr + hlen);
						break;
				default:
						printf("packet contains an unknown protocol. skipping...");
						checkcount();
						break;
		}
		
		return;
}

void *tcp_process(char *ptr) {
		struct tcphdr *tcp;
		struct servent *sptr;
		
		tcp = (struct tcphdr *) ptr;
		printf("\nProtocol: TCP. PORTS: ");
    if (names) {
				if ((sptr = getservbyport(tcp->source, "tcp")) == NULL)
						printf("%d >> ", ntohs(tcp->source));
				else
						printf("%s >> ", sptr->s_name);
				if ((sptr = getservbyport(tcp->dest, "tcp")) == NULL)
						printf("%d , ", ntohs(tcp->dest));
				else
						printf("%s , ", sptr->s_name);
		} else																														
		printf("%d >> %d, ", ntohs(tcp->source), ntohs(tcp->dest));
		
		if (tcp->ack == 1)
				printf("ACK, ");
		if (tcp->syn == 1)
				printf("SYN, ");
		if (tcp->fin == 1)
				printf("FIN, ");
		if (tcp->rst == 1)
				printf("RST, ");
		if (tcp->psh == 1)
				printf("PSH, ");
		if (tcp->urg == 1)
				printf("URG, ");
		
		if (verbose)
				printf("SEQ: %d, ACK_SEQ: %d, ", ntohs(tcp->seq), ntohs(tcp->ack_seq));
		printf("WINDOW: %d", ntohs(tcp->window));
		checkport(ntohs(tcp->source), ntohs(tcp->dest), ptr+sizeof(struct tcphdr));
		
		return;
}

void udp_process(char *ptr) {
		struct udphdr *udp;
		struct servent *sptr;
		
		udp = (struct udphdr *) ptr;
		printf("\nProtocol: UDP. ports: ");
		if (names) {
				if ((sptr = getservbyport(udp->source, "udp")) == NULL) {
						printf("%d >> ", ntohs(udp->source));
				} else 
						printf("%s >> ", sptr->s_name);
				if ((sptr = getservbyport(udp->dest, "udp")) == NULL)
						printf("%d ,", ntohs(udp->dest));
				else
						printf("%s ,", sptr->s_name);
		} else 
				printf("%d >> %d, ", ntohs(udp->source), ntohs(udp->dest));
		printf(" udp-legth: %d", ntohs(udp->len));
		checkport(ntohs(udp->source), ntohs(udp->dest), ptr+sizeof(struct udphdr));
		
		return;
}

void icmp_process(char *ptr) {
		struct icmp *icmp;
		
		icmp = (struct icmp *) ptr;
		printf("\nProtocol: ICMP.");
		switch (icmp->icmp_type) {
				case ICMP_ECHOREPLY:
						printf(" Echo-reply (type %d).", icmp->icmp_type);
						break;
				case ICMP_DEST_UNREACH:
						printf(" Destination-unreachable (type %d):", icmp->icmp_type);
						switch (icmp->icmp_code) {
								case ICMP_NET_UNREACH:
										printf("network unreachable (code %d).", icmp->icmp_code);
										break;
								case ICMP_HOST_UNREACH:
										printf("host unreachable (code %d).", icmp->icmp_code);
										break;
								case ICMP_PROT_UNREACH:
										printf("protocol unreachable (code %d).", icmp->icmp_code);
										break;
								case ICMP_PORT_UNREACH:
										printf("port unreachable (code %d).", icmp->icmp_code);
										break;
								case ICMP_FRAG_NEEDED:
										printf("fragmentation needed (code %d).", icmp->icmp_code);
										break;
								case ICMP_SR_FAILED:
										printf("source-route failed (code %d).", icmp->icmp_code);
										break;
								case ICMP_NET_UNKNOWN:
										printf("unknown network (code %d).", icmp->icmp_code);
										break;
								case ICMP_HOST_UNKNOWN:
										printf("unknown host (code %d).", icmp->icmp_code);
										break;
								case ICMP_HOST_ISOLATED:
										printf("host isolated (code %d).", icmp->icmp_code);
										break;
								case ICMP_NET_ANO:
										break;
								case ICMP_HOST_ANO:
										break;
								case ICMP_NET_UNR_TOS:
										printf("network unreachable for type-of-service (code %d).", icmp->icmp_code);
										break;
								case ICMP_HOST_UNR_TOS:
										printf("host unreachable for type-of-service (code %d).", icmp->icmp_code);
										break;
								case ICMP_PKT_FILTERED:
										printf("packet filtered (code %d).", icmp->icmp_code);
										break;
								case ICMP_PREC_VIOLATION:
										printf("precedence violation (code %d).", icmp->icmp_code);
										break;
								case ICMP_PREC_CUTOFF:
										printf("precedence cutoff (code %d).", icmp->icmp_code);
										break;
						}
						break;
				case ICMP_SOURCE_QUENCH:
						printf(" Source quench  (type %d).", icmp->icmp_type);
						break;
				case ICMP_REDIRECT:
						printf("redirect (type %d) for: ", icmp->icmp_type);
						switch (icmp->icmp_code) {
								case ICMP_REDIR_NET:
										printf("network (code %d).", icmp->icmp_code);
										break;
								case ICMP_REDIR_HOST:
										printf("host (code %d).", icmp->icmp_code);
										break;
								case ICMP_REDIR_NETTOS:
										printf("network for type-of-service (code %d).", icmp->icmp_code);
										break;
								case ICMP_REDIR_HOSTTOS:
										printf("host for type-of-service (code %d).", icmp->icmp_code);
										break;
						}
						break;
				case ICMP_ECHO:
						printf(" Echo-request (type %d).", icmp->icmp_type);
						break;
				case ICMP_ROUTERSOLICIT:
						printf(" Router-Solicitation (type %d).", icmp->icmp_type);
						break;
				case ICMP_ROUTERADVERT:
						printf(" Router-Advertisement (type %d).", icmp->icmp_type);
						break;
				case ICMP_TIME_EXCEEDED:
						printf(" Time-exceeded (type %d):", icmp->icmp_type);
						switch (icmp->icmp_code) {
								case ICMP_EXC_TTL:
										printf("time-to-live count exceeded.");
										break;
								case ICMP_EXC_FRAGTIME:
										printf("fragment reass time exceeded.");
										break;
						}
						break;
				case ICMP_PARAMETERPROB:
						printf(" Parameter-problem (type %d).", icmp->icmp_type);
						break;
				case ICMP_TIMESTAMP:
						printf(" Timestamp-request (type %d).", icmp->icmp_type);
						break;
				case ICMP_TIMESTAMPREPLY:
						printf(" Timestamp-reply (type %d).", icmp->icmp_type);
						break;
				case ICMP_INFO_REQUEST:
						printf(" Info-request (type %d).", icmp->icmp_type);
						break;
				case ICMP_INFO_REPLY:
						printf(" Info-reply (type %d).", icmp->icmp_type);
						break;
				case ICMP_ADDRESS:
						printf(" Address-request (type %d).", icmp->icmp_type);
						break;
				case ICMP_ADDRESSREPLY:
						printf(" Address-reply (type %d).", icmp->icmp_type);
						printf(" Address mask is 0x%08x.", ntohl(icmp->icmp_mask));
						break;
		}
		if (verbose)
				printf(" ICMP-ID: %d; ICMP SEQ: %d.", icmp->icmp_id, icmp->icmp_seq);
		checkcount();

		return;
}

void arp_process(char *ptr, uint8_t sender[ETH_ALEN], uint8_t target[ETH_ALEN]) {
		struct ether_arp *earp;
		int i;

		earp = (struct ether_arp *) ptr;
		switch (ntohs(earp->ea_hdr.ar_op)) {
				case ARPOP_REQUEST:
						printf("ARP-REQUEST: who has %d.%d.%d.%d tell ", earp->arp_tpa[0], earp->arp_tpa[1],
										earp->arp_tpa[2], earp->arp_tpa[3]);
						for (i=0; i<4; i++) printf("%d.", earp->arp_spa[i]);
						break;
				case ARPOP_REPLY:
						printf("ARP-REPLY: %d.%d.%d.%d is at ", earp->arp_spa[0], earp->arp_spa[1],
										earp->arp_spa[2], earp->arp_spa[3]);
						switch (ntohs(earp->ea_hdr.ar_hrd)) {
								case ARPHRD_ETHER:
										print_etheradr(earp->arp_sha);
										break;
						}
		}
		if (verbose) {
				switch (ntohs(earp->ea_hdr.ar_hrd)) {
						case ARPHRD_ETHER:
								printf(" (");
								print_hadrs(earp->arp_sha, earp->arp_tha);
								printf(")");
								break;
				}
		}				
		checkcount();

		return;
}

